package ms.itsecteam.warpdrink.manager;

import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.view.WindowManager;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.AutoCompleteTextView;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.ListView;
import android.widget.TextView;

import java.util.ArrayList;
import java.util.List;

import ms.itsecteam.warpdrink.MainActivity;
import ms.itsecteam.warpdrink.R;
import ms.itsecteam.warpdrink.data.models.local.Product;
import ms.itsecteam.warpdrink.data.models.local.User;
import ms.itsecteam.warpdrink.dialogs.ChargeDialog;
import ms.itsecteam.warpdrink.dialogs.NewUserDialog;
import ms.itsecteam.warpdrink.orderList.ListViewAdapter;
import ms.itsecteam.warpdrink.orderList.ListViewAdapterProducts;
import ms.itsecteam.warpdrink.orderList.Order;

public class UiManager implements View.OnKeyListener, View.OnClickListener, AdapterView.OnItemClickListener {

    private MainActivity mainActivity;
    private Button btnPay, btnCharge, btnLogout;
    private ImageButton btnRefresh;
    private TextView txtSum,txtCredit;
    private ListView lstOrdered;
    private ListView lstProduct;
    private AutoCompleteTextView atxvName;

    private ListViewAdapter lstOrderedAdapter;
    private ListViewAdapterProducts lstProductAdapter;
    private ArrayAdapter atxvAdapter;
    private ArrayList<Double> order;
    private List<User> users;

    private static UiManager ourInstance = new UiManager();

    public static UiManager getInstance() {
        return ourInstance;
    }

    private UiManager() {}

    protected void initUi(MainActivity mainActivity) {
        this.txtSum = (TextView) mainActivity.findViewById(R.id.txtSum);
        this.txtCredit = (TextView)mainActivity.findViewById(R.id.txtCredit);
        this.txtCredit.setText(mainActivity.getResources().getString(R.string.credit,0.0));
        this.txtSum.setText(mainActivity.getResources().getString(R.string.total,0.0));

        this.btnLogout = (Button) mainActivity.findViewById(R.id.btnLogout);
        this.btnRefresh = (ImageButton) mainActivity.findViewById(R.id.btnRefresh);
        this.btnPay = (Button) mainActivity.findViewById(R.id.btnPay);
        this.btnCharge = (Button) mainActivity.findViewById(R.id.btnCharge);

        this.lstOrderedAdapter = new ListViewAdapter(mainActivity, R.layout.order_list_item, new ArrayList<Order>());
        this.lstProductAdapter = new ListViewAdapterProducts(mainActivity, R.layout.product_list_item, Product.getAll());

        this.lstOrdered = (ListView) mainActivity.findViewById(R.id.lstOrdered);
        this.lstOrdered.setAdapter(this.lstOrderedAdapter);

        this.lstProduct = (ListView) mainActivity.findViewById(R.id.lstProduct);
        this.lstProduct.setAdapter(this.lstProductAdapter);
        this.lstProduct.setOnItemClickListener(this);
        this.users = User.getAll();
        this.atxvAdapter = new ArrayAdapter(mainActivity, android.R.layout.select_dialog_item, this.users);
        this.atxvName = (AutoCompleteTextView) mainActivity.findViewById(R.id.atxvName);
        this.atxvName.setAdapter(atxvAdapter);
        this.atxvName.setThreshold(1);
        this.atxvName.setOnKeyListener(this);
        this.atxvName.setOnItemClickListener(this);
        this.atxvName.setFocusableInTouchMode(true);
        this.atxvName.setFocusable(true);

        this.btnLogout.setVisibility(View.INVISIBLE);
        this.enableControls(false);
        this.mainActivity = mainActivity;
    }

    protected void refreshTotalTextView(double totalAmount) {
        this.txtSum.setText(this.mainActivity.getResources().getString(R.string.total,totalAmount));
    }
    protected void refreshCreditTextView(double credit) {
        this.txtCredit.setText(this.mainActivity.getResources().getString(R.string.credit, credit));
    }

    protected void setCurrentUser(User user) {
        this.enableControls(true);
        this.atxvName.setText(user.getUserid());
        this.atxvName.dismissDropDown();
        this.btnLogout.setVisibility(View.VISIBLE);
        this.refreshCreditTextView(user.getCredit());
    }

    protected void clearCurrentUser() {
        this.atxvName.setText("");
        if(this.atxvName.requestFocus()) {
            this.mainActivity.getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_VISIBLE);
        }
        this.lstOrderedAdapter.clear();
        this.enableControls(false);
        this.btnLogout.setVisibility(View.INVISIBLE);
        this.refreshCreditTextView(0.0);
        this.refreshTotalTextView(0.0);
    }

    private void enableControls(boolean enable) {
        this.atxvName.setEnabled(!enable);
        this.btnRefresh.setEnabled(!enable);
        this.btnCharge.setEnabled(enable);
        this.btnPay.setEnabled(enable);
        this.btnLogout.setEnabled(enable);
        this.lstProduct.setEnabled(enable);
    }

    protected void refreshUserData(List<User> userList) {
        this.atxvAdapter.clear();
        for (User anUserList : userList) {
            this.atxvAdapter.add(anUserList);
        }
    }
    public void refreshProductData(List<Product> productList) {
        this.lstProductAdapter.clear();
        this.lstProductAdapter.addAll(productList);
    }

    protected void removeOrder(Order order) {
        this.lstOrderedAdapter.remove(order);
    }

    @Override
    public void onClick(View v) {
        Log.d("HALLO", Integer.toString(v.getId()));
        switch (v.getId()) {
         /*   case R.id.btnOneEuro:
                this.lstOrderedAdapter.insert(new Order(1.0), this.lstOrderedAdapter.getCount());
                MainManager.getInstance().addOrder(1.0);
                break;
            case R.id.btnFiftyCent:
                this.lstOrderedAdapter.insert(new Order(0.5), this.lstOrderedAdapter.getCount());
                MainManager.getInstance().addOrder(0.5);
                break;*/
            case R.id.btnCharge:
                new ChargeDialog().show();
                break;
            case R.id.btnPay:
                MainManager.getInstance().performPayment();
                break;
            case R.id.btnLogout:
                MainManager.getInstance().clearCurrentUser();
                break;
            case R.id.btnAddUser:
                new NewUserDialog().show();
                break;
            case R.id.btnRefresh:
                MainManager.getInstance().refreshData();
                break;
        }
    }

    @Override
    public boolean onKey(View v, int keyCode, KeyEvent event) {
        if (event.getAction()==KeyEvent.ACTION_DOWN && event.getKeyCode() == KeyEvent.KEYCODE_ENTER) {
            if(this.atxvAdapter.getCount() == 1) {
                MainManager.getInstance().setCurrentUser((User) this.atxvAdapter.getItem(0));
                this.atxvName.setText(this.atxvAdapter.getItem(0).toString());
                this.atxvName.dismissDropDown();
            }
            return true;
        }
        return false;
    }
    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
        switch (parent.getId()) {
            case R.id.lstProduct:
                Product p = this.lstProductAdapter.getItem(position);
                Order order = new Order(p);
                this.lstOrderedAdapter.insert(new Order(p), this.lstOrderedAdapter.getCount());
                MainManager.getInstance().addOrder(order);
                break;
            case -1:
                MainManager.getInstance().setCurrentUser((User) this.atxvAdapter.getItem(position));
            break;
        }
    }


}
